#!/bin/sh

## global parameters ##
# role auto-detection
role_file_path="/tmp/currentrole"
currentrole="client"
[ -e "$role_file_path" ] && currentrole=$(cat $role_file_path)
ping_retry_limit=3
debugMsgEnable=0
batman_gw_role=$(batctl gw | awk '{print $1}')

# root hop count
hop_count_file="/tmp/mesh_root_hc"

# renew lan ip
gw_mac_file="/tmp/mesh_gw_mac"

## sub function ##
renewLanIp()
{
    local proto=$(uci get network.lan.proto)
    local PID

    case $proto in
        dhcp)
            PID=`pidof udhcpc`
            #1. release
            /bin/kill -SIGUSR2 $PID
            #2. renew
            /bin/kill -SIGUSR1 $PID
            ;;
        *)
            ;;
    esac
}

chkRenewLanIp()
{
    local gw_mac="$(batctl gwl | grep "=>" | awk -F" " '{ print $2 }')"
    [ -z "$gw_mac" -a -e "/tmp/masterMac" ] && gw_mac=`cat /tmp/masterMac`
    
    local pre_gw_mac

    if [ -e "$gw_mac_file" ]; then
        pre_gw_mac=$(cat $gw_mac_file)
    fi

    if [ -z "$pre_gw_mac" ]; then
        echo $gw_mac > $gw_mac_file
        pre_gw_mac=$gw_mac
    fi

    if [ "$gw_mac" != "$pre_gw_mac" ]; then
        echo $gw_mac > $gw_mac_file
        renewLanIp
    fi
}

calRootCount()
{
    local hop=-1

    #20180331 Jason:Fix for BATMAN2017
    local batman_ver=`batctl o |grep 2016 1>/dev/null 2>/dev/null && echo '2016' || echo '2017up'`

    if [ "$batman_ver" = "2016" ]; then
        local gw_mac="$(batctl gwl | grep "=>" | awk -F" " '{ print $2 }')"
    else
        local gw_mac="$(batctl gwl | grep '*' | awk -F" " '{ print $2 }')"
    fi
    [ -z "$gw_mac" -a -e "/tmp/masterMac" ] && gw_mac=`cat /tmp/masterMac`

    if [ "$debugMsgEnable" == "1" ]; then
	echo "debug gw_mac:[$gw_mac]"
    fi
    if [ -n "$gw_mac" ]; then
        hop="$(batctl tr $gw_mac | wc -l)"
    fi
    if [ "$debugMsgEnable" == "1" ]; then	
	echo "debug hop:[$hop]"
    fi
    if [ "$hop" != "0" -a "$hop" != "-1" ]; then
        let "hop=$hop - 1"
    fi
    echo "$hop" > "$hop_count_file"
}

slaveServerMeshSync()
{
	[ ! -e "/tmp/mesh_sync_client_apply_data" ] && {
		echo "[chkEth2GW] connected to master, ask for sync" > /dev/console
		meshif=`getinfo mesh_ifname`
		local mac=`ifconfig $meshif|grep HWaddr|cut -d" " -f10|tr A-Z a-z`
		masterMac=$(batctl gwl -H | grep "=>" | awk '{print $2}' | grep -E "^[a-f0-9:]{17}$")
	    	[ -z "$masterMac" -a -e "/tmp/masterMac" ] && masterMac=`cat /tmp/masterMac`
		forceSync=0
		[ -e /tmp/ezsetup_led ] && forceSync=1
		mesh_control "mesh_sync sync_server_conf $mac $forceSync" $masterMac
	}
}

setMeshAP()
{
    if [ -f "/etc/config/ezmcloud" -a -f "/sbin/mesh_rssi_led.sh" ]; then
        local PID=$(ps | grep mesh_rssi_led.sh | grep -v "grep" | awk '{print $1}')
        if [ -z "$PID" ]; then
            /sbin/mesh_rssi_led.sh &
        fi
    fi

    if [ "$currentrole" == "client" -a "$batman_gw_role" == "client" ]; then
        # renew lan ip
        chkRenewLanIp
        # root hop count
        calRootCount
        exit 0
    fi
    batctl gw client
    echo client > $role_file_path

    enable="$(uci show wireless | grep l2_isolatior=\'1\' | wc -l)"
    if [ "$enable" != "0" ]; then
	luci-reload auto l2_isolation
    fi

    uci set mesh.wifi.role=client
    uci commit mesh

    # renew lan ip
    chkRenewLanIp
    # root hop count
    calRootCount

    if [ "$(uci -q get mesh.wifi.supportRP)" == "1" -a "$(uci get mesh.wifi.disabled)" == "0" ]; then
        if [ -n "$(pidof wpa_cli)" ]; then
            sta_if="ath29"
            [ "$(uci get wireless.wifi1_mesh.disabled)" == "0" ] && sta_if="ath59"
            sed -i "/bssid=/d" /var/run/wpa_supplicant-${sta_if}.conf
            sed -i "/key_mgmt=.*/i bssid=12:34:56:78:90:00" /var/run/wpa_supplicant-${sta_if}.conf
            wpa_cli -i${sta_if} -p/var/run/wpa_supplicant-${sta_if} reconfigure
        fi

        sleep 5; sh /usr/sbin/setRPMac chkEth2GW
    fi
}

setGatewayAP()
{
    if [ -f "/etc/config/ezmcloud" -a -f "/sbin/mesh_rssi_led.sh" ]; then
        local PID=$(ps | grep mesh_rssi_led.sh | grep -v "grep" | awk '{print $1}')
        if [ -n "$PID" ]; then
            kill -9 $PID
            /usr/shc/led start
        fi
    fi

    if [ "$(uci -q get mesh.wifi.supportRP)" == "1" -a "$(uci get mesh.wifi.disabled)" == "0" ]; then
        sta_if="ath29"
        [ "$(uci get wireless.wifi1_mesh.disabled)" == "0" ] && sta_if="ath59"
        wpa_cli -i${sta_if} -p/var/run/wpa_supplicant-${sta_if} disconnect
    fi

    echo 0 > "$hop_count_file"
    if [ "$currentrole" == "server" -a "$batman_gw_role" == "server" ]; then
        exit 0
    fi
    batctl gw server
    echo server > $role_file_path

    enable="$(uci show wireless | grep l2_isolatior=\'1\' | wc -l)"

    if [ "$enable" != "0" ]; then
	luci-reload auto l2_isolation
    fi

    uci set mesh.wifi.role=server
    uci commit mesh

    if [ "$(uci -q get mesh.wifi.supportRP)" == "1" -a "$(uci get mesh.wifi.disabled)" == "0" ]; then
        if [ -n "$(cat /etc/crontabs_nolog/root |grep setRPMac)" ]; then
            sed -i '/setRPMac/d' /etc/crontabs_nolog/root
            crontab -c /etc/crontabs_nolog /etc/crontabs_nolog/root
        fi
    fi
}

## main function ##

# [ ! -e "/tmp/booting_is_done" ] && exit

mesh24GDisabled=$(uci get wireless.wifi0_mesh.disabled)
mesh5GDisabled=$(uci get wireless.wifi1_mesh.disabled)

if [ "$mesh24GDisabled" == 1 -a "$mesh5GDisabled" == 1 ]; then
	if [ "$debugMsgEnable" == "1" ]; then
		echo ===Mesh function is disabled, clean checking function in crontab=== >> /dev/console
	fi
    sed -i '/chkEth2GW/d' /etc/crontabs/root
    crontab /etc/crontabs/root
    exit 0
fi

gatewayIP=$(route -n| grep 'UG[ \t]'|awk '{ print $2 }')

if [ -z "$gatewayIP" ] || [ "$gatewayIP" == "0.0.0.0" ]; then
	if [ "$debugMsgEnable" == "1" ]; then
		echo ===error! device does not have gateway ip==== >> /dev/console
	fi
	setMeshAP
	exit 0
fi

opmode=$(uci get /etc/config/system.@system[0].opmode)

if [ "ar" == "$opmode" ]; then
	wan_proto=$(uci get /etc/config/network.wan.proto)

	if [ "pppoe" == "$wan_proto" ]; then
		#devIP=$(ifconfig pppoe-wan | awk '/inet addr/{print substr($2,6)}')
		devIP=$(ubus call network.interface.wan status | grep address | awk -F'"' '{print $4}')

		if [ -n "$devIP" ]; then
			setGatewayAP
			exit 0
		fi
	fi
	ethInterfaceList=$(uci get /etc/config/network.wan.ifname)
	devIP=$(ifconfig $ethInterfaceList | awk '/inet addr/{print substr($2,6)}')
else
	devIP=$(ifconfig br-lan | awk '/inet addr/{print substr($2,6)}')
	ethInterfaceList=$(uci get /etc/config/network.lan.ifname)
fi

if [ -z "$devIP" ]; then
	if [ "$debugMsgEnable" == "1" ]; then
	        echo ===error! device does not have ip address=== >> /dev/console
	fi
	setMeshAP
    exit 0
fi

gatewayMac=$(cat /proc/net/arp | grep "$gatewayIP " | grep -v 00:00:00:00:00:00 | awk '{print $4}')

if [ -z "$gatewayMac" ]; then		#arping the gateway mac to refresh the arp table
	arping -I br-lan $gatewayIP -c 1
	gatewayMac=$(cat /proc/net/arp | grep "$gatewayIP " | grep -v 00:00:00:00:00:00 | awk '{print $4}')
fi

#interface_arping -I $ethInterface -S $devIP -T $gatewayIP -x

for ethInterface in $ethInterfaceList
do
	# check ethxx
	if [ ${ethInterface#eth} != ${ethInterface} ];then
		chkEth=$(cat /sys/class/net/"$ethInterface"/carrier)
		# link down
		if [ "$chkEth" != "1" ]; then
			if [ "$debugMsgEnable" == "1" ]; then
				echo ===ethernet not plugged in, set mesh ap directly=== >> /dev/console
			fi
			continue
		fi

		for i in `seq 1 $ping_retry_limit`
		do
			if [ -z "$gatewayMac" ]; then
				pingAns=$(interface_ping -I $ethInterface -S $devIP -T $gatewayIP -d 1 | grep "bytes received" | awk '{print $1}')
			else
				pingAns=$(interface_ping -I $ethInterface -S $devIP -T $gatewayIP -d 1 -M $gatewayMac | grep "bytes received" | awk '{print $1}')
			fi

			if [ "$pingAns" == "$gatewayIP" ]; then
				if [ "$debugMsgEnable" == "1" ]; then
					echo pingAns=$pingAns gatewayIP=$gatewayIP >> /dev/console
				fi
				setGatewayAP
				exit 0
			fi

			[ -f /bin/interface_arping ] && {
				pingAns=$(interface_arping -I $ethInterface -S $devIP -T $gatewayIP -d 1 -a | grep "received arp" | awk '{print $4}')
				if [ "$pingAns" == "$gatewayIP" ]; then
					if [ "$debugMsgEnable" == "1" ]; then
						echo pingAns=$pingAns gatewayIP=$gatewayIP >> /dev/console
					fi
					setGatewayAP
					exit 0
				fi
			}

			if [ "$debugMsgEnable" == "1" ]; then
				echo ping failed $i times >> /dev/console
			fi
		done
	fi
done

setMeshAP

